#!/bin/sh
#
# Control script for IP pool
#

PKG_USERCONF_DIR="/usr/syno/etc/packages/VPNPlusServer"
PKG_TARGET_DIR="/var/packages/VPNPlusServer/target"
SZF_VPNPLUS_CONF="${PKG_USERCONF_DIR}/synovpnplus.conf"
SZF_OBJECTS_CONF="${PKG_USERCONF_DIR}/vpnplus_objects.conf"
SZF_VPNTOOL="${PKG_TARGET_DIR}/tool/synovpnplustool"
BIN_SYNOMODULETOOL="/usr/syno/bin/synomoduletool"
SZF_JQ="/bin/jq"
SZF_GET_KEY_VALUE="/bin/get_key_value"
SZF_GET_SECTION_KEY_VALUE="/usr/syno/bin/get_section_key_value"
SERVICE="vpnplus"
TOPOLOGY=`/bin/get_key_value /etc/synoinfo.conf net_topology`

start() {
	bridgeIntf=$1
	key=${bridgeIntf#vbr*}
	tapIntf="tap_"$1
	serverIP=`${SZF_JQ} -r --arg key $key --arg value server_ip 'to_entries[] | select(.key == $key) | .value[$value]' ${SZF_OBJECTS_CONF}`
	netMask=`${SZF_JQ} -r --arg key $key --arg value netmask 'to_entries[] | select(.key == $key) | .value[$value]' ${SZF_OBJECTS_CONF}`
	enable_ip_assignment=`${SZF_JQ} -r --arg key $key --arg value enable_ip_assignment 'to_entries[] | select(.key == $key) | .value[$value]' ${SZF_OBJECTS_CONF}`
	ifcfg_brige="/etc/sysconfig/network-scripts/ifcfg-${bridgeIntf}"
	ifcfg_tap="/etc/sysconfig/network-scripts/ifcfg-${tapIntf}"

	if [ "yes" = "${enable_ip_assignment}" ]; then
		if [ -x ${BIN_SYNOMODULETOOL} ]; then
			$BIN_SYNOMODULETOOL --insmod $SERVICE tun.ko
		fi

		brctl addbr ${bridgeIntf}
		ip tuntap add dev ${tapIntf} mode tap
		brctl addif ${bridgeIntf} ${tapIntf}
		ifconfig ${tapIntf} up
		ifconfig ${bridgeIntf} ${serverIP} netmask ${netMask} up

		# write ifcfg
		${SZF_VPNTOOL} -u -s ${ifcfg_brige} DEVICE ${bridgeIntf}
		${SZF_VPNTOOL} -u -s ${ifcfg_brige} PRIMARY ${tapIntf}
		${SZF_VPNTOOL} -u -s ${ifcfg_brige} TYPE bridge
		${SZF_VPNTOOL} -u -s ${ifcfg_brige} LOCAL_LAN yes
		${SZF_VPNTOOL} -u -s ${ifcfg_brige} BOOTPROTO static
		${SZF_VPNTOOL} -u -s ${ifcfg_brige} IPADDR ${serverIP}
		${SZF_VPNTOOL} -u -s ${ifcfg_brige} NETMASK ${netMask}
		${SZF_VPNTOOL} -u -s ${ifcfg_brige} ONBOOT no

		${SZF_VPNTOOL} -u -s ${ifcfg_tap} BRIDGE ${bridgeIntf}
		${SZF_VPNTOOL} -u -s ${ifcfg_tap} DEVICE ${tapIntf}
		${SZF_VPNTOOL} -u -s ${ifcfg_tap} BOOTPROTO static
		${SZF_VPNTOOL} -u -s ${ifcfg_tap} ONBOOT no

		iptables -A VPN_SERVER_INPUT -i ${bridgeIntf} -j SYNO_INPUT_ACCEPT
	fi
}

stop() {
	bridgeIntf=$1
	key=${bridgeIntf#vbr*}
	tapIntf="tap_"$1
	enable_ip_assignment=`${SZF_JQ} -r --arg key $key --arg value enable_ip_assignment 'to_entries[] | select(.key == $key) | .value[$value]' ${SZF_OBJECTS_CONF}`
        ifcfg_brige="/etc/sysconfig/network-scripts/ifcfg-${bridgeIntf}"
	ifcfg_tap="/etc/sysconfig/network-scripts/ifcfg-${tapIntf}"

	if [ "yes" = "${enable_ip_assignment}" ]; then
		ifconfig ${tapIntf} down
		brctl delif ${bridgeIntf} ${tapIntf}
		ip tuntap del dev ${tapIntf} mode tap
		ifconfig ${bridgeIntf} down
		brctl delbr ${bridgeIntf}

		/bin/rm -f ${ifcfg_brige}
		/bin/rm -f ${ifcfg_tap}

		if [ -x ${BIN_SYNOMODULETOOL} ]; then
			$BIN_SYNOMODULETOOL --rmmod $SERVICE tun.ko
		fi

		iptables -D VPN_SERVER_INPUT -i ${bridgeIntf} -j SYNO_INPUT_ACCEPT
	fi
}

restart() {
	bridgeIntf=$1
	key=${bridgeIntf#vbr*}
	tapIntf="tap_"$1
	serverIP=`${SZF_JQ} -r --arg key $key --arg value server_ip 'to_entries[] | select(.key == $key) | .value[$value]' ${SZF_OBJECTS_CONF}`
	netMask=`${SZF_JQ} -r --arg key $key --arg value netmask 'to_entries[] | select(.key == $key) | .value[$value]' ${SZF_OBJECTS_CONF}`
	ifcfg_brige="/etc/sysconfig/network-scripts/ifcfg-${bridgeIntf}"

	ifconfig ${bridgeIntf} ${serverIP} netmask ${netMask}

	# write ifcfg
	${SZF_VPNTOOL} -u -s ${ifcfg_brige} IPADDR ${serverIP}
	${SZF_VPNTOOL} -u -s ${ifcfg_brige} NETMASK ${netMask}
}

startTap() {
	# add tap_$1
	bridgeIntf=$1
	tapIntf="tap_"$1
	enable=`${SZF_GET_KEY_VALUE} /etc/dhcpd/dhcpd-${bridgeIntf}-${bridgeIntf}0.info enable`
	ifcfg="/etc/sysconfig/network-scripts/ifcfg-${tapIntf}"

	if [ "yes" = "${enable}" ]; then
		ip tuntap add dev ${tapIntf} mode tap
		brctl addif ${bridgeIntf} ${tapIntf}
		ifconfig ${tapIntf} up
		${SZF_VPNTOOL} -u -s ${ifcfg} BRIDGE ${bridgeIntf}
		${SZF_VPNTOOL} -u -s ${ifcfg} DEVICE ${tapIntf}
		${SZF_VPNTOOL} -u -s ${ifcfg} BOOTPROTO static
		${SZF_VPNTOOL} -u -s ${ifcfg} ONBOOT no
	fi
}

stopTap() {
	# del tap_$1
	bridgeIntf=$1
	tapIntf="tap_"$1
	ifcfg="/etc/sysconfig/network-scripts/ifcfg-${tapIntf}"

	ifconfig ${tapIntf} down
	brctl delif ${bridgeIntf} ${tapIntf}
	ip tuntap del dev ${tapIntf} mode tap
	/bin/rm -f ${ifcfg}
}

startAll() {
	used_interface=`${SZF_GET_SECTION_KEY_VALUE} ${SZF_VPNPLUS_CONF} ippool used_interface`

	if [ -x ${BIN_SYNOMODULETOOL} ]; then
		$BIN_SYNOMODULETOOL --insmod $SERVICE tun.ko
	fi

	#$for i in `echo ${used_interface} | grep -o "1" | awk -F ":" '{print $1}'`; do
	for i in `seq 3 $((${#used_interface}-1))`; do
		if [ "1" = "${used_interface:$i:1}" ]; then
			start "vbr$i"
		fi
	done

	# add tap_lbr0 and tap_gbr0
	startTap lbr0
	startTap gbr0

	# add SNAT in bridge mode
	if [ "bridge" = "${TOPOLOGY}" ]; then
		addAllSNAT
	fi

	/etc/rc.network nat-restart-dhcp
}

stopAll() {
	used_interface=`${SZF_GET_SECTION_KEY_VALUE} ${SZF_VPNPLUS_CONF} ippool used_interface`
	#$for i in `echo ${used_interface} | grep -o "1" | awk -F ":" '{print $1}'`; do
	for i in `seq 3 $((${#used_interface}-1))`; do
		if [ "1" = "${used_interface:$i:1}" ]; then
			stop "vbr$i"
		fi
	done

	# del tap_lbr0 and tap_gbr0
	stopTap lbr0
	stopTap gbr0

	# del SNAT in bridge mode
	if [ "bridge" = "${TOPOLOGY}" ]; then
		delAllSNAT
	fi

	/etc/rc.network nat-restart-dhcp

	if [ -x ${BIN_SYNOMODULETOOL} ]; then
		$BIN_SYNOMODULETOOL --rmmod $SERVICE tun.ko
	fi
}

addAllSNAT() {
	used_interface=`${SZF_GET_SECTION_KEY_VALUE} ${SZF_VPNPLUS_CONF} ippool used_interface`
	for i in `seq 3 $((${#used_interface}-1))`; do
		if [ "1" = "${used_interface:$i:1}" ]; then
			enable_ip_assignment=`${SZF_JQ} -r --arg key $i --arg value enable_ip_assignment 'to_entries[] | select(.key == $key) | .value[$value]' ${SZF_OBJECTS_CONF}`
			if [ "yes" = ${enable_ip_assignment} ]; then
				addSNAT "$i"
			fi
		fi
	done
}

addSNAT() {
	if [ "bridge" != "${TOPOLOGY}" ]; then
		return
	fi

	cidr=`${SZF_JQ} -r --arg key $1 --arg value cidr 'to_entries[] | select(.key == $key) | .value[$value]' ${SZF_OBJECTS_CONF}`
	/sbin/iptables -t nat -A POSTROUTING -s ${cidr} -j MASQUERADE
}

delAllSNAT() {
	used_interface=`${SZF_GET_SECTION_KEY_VALUE} ${SZF_VPNPLUS_CONF} ippool used_interface`
	for i in `seq 3 $((${#used_interface}-1))`; do
		if [ "1" = "${used_interface:$i:1}" ]; then
			enable_ip_assignment=`${SZF_JQ} -r --arg key $i --arg value enable_ip_assignment 'to_entries[] | select(.key == $key) | .value[$value]' ${SZF_OBJECTS_CONF}`
			if [ "yes" = ${enable_ip_assignment} ]; then
				delSNAT "$i"
			fi
		fi
	done
}

delSNAT() {
	if [ "bridge" != "${TOPOLOGY}" ]; then
		return
	fi

	cidr=`${SZF_JQ} -r --arg key $1 --arg value cidr 'to_entries[] | select(.key == $key) | .value[$value]' ${SZF_OBJECTS_CONF}`
	/sbin/iptables -t nat -D POSTROUTING -s ${cidr} -j MASQUERADE
}

case "$1" in
	start)
		start "$2"
	;;
	stop)
		stop "$2"
	;;
	restart)
		restart "$2"
	;;
	startTap)
		startTap "$2"
	;;
	stopTap)
		stopTap "$2"
	;;
	start-all)
		startAll "$@"
	;;
	stop-all)
		stopAll "$@"
	;;
	add-all-snat)
		addAllSNAT
	;;
	add_snat)
		addSNAT "$2"
	;;
	del-all-snat)
		delAllSNAT
	;;
	del_snat)
		delSNAT "$2"
	;;
	*)
		echo "Usage: $0 {start <interface>|stop <interface>|restart <interface>|startTap <interface>|stopTap <interface>|start-all|stop-all|add-all-snat|del-all-snat|add_snat <object_id>|del_snat <object_id>}"
		exit 1
	esac
exit 0
