#!/bin/sh

#################################################################
# This hook is invoked when ip change
# For renew push "route {private network}"
# Usable environment variable: IFNAME,ORIGIN_ADDRESS,NEW_ADDRESS
#################################################################

PACKAGE_NAME="VPNPlusServer"
PACKAGE_TARGET_DIR="/var/packages/${PACKAGE_NAME}/target"
PACKAGE_USR_CONF_DIR="/var/packages/${PACKAGE_NAME}/etc"
VPNPLUS_CONFIG="${PACKAGE_USR_CONF_DIR}/synovpnplus.conf"
OPENVPN_CONFIG="${PACKAGE_USR_CONF_DIR}/openvpn/openvpn.conf"

modify_push_route () {
	if [ -z "${IFNAME}" ]; then
		return
	fi

	local ServerInfo=`/bin/cat ${OPENVPN_CONFIG} | /bin/grep ^server | /usr/bin/awk '{ print $2 }'`
	local OldRoute=`/bin/grep route ${OPENVPN_CONFIG} | /bin/grep -v ${ServerInfo}`
	local CurIP=`/sbin/ip addr show ${IFNAME} | /bin/grep "inet" | /bin/grep -v "inet6" | /usr/bin/awk '{ print $2 }'`

	local NewNet=`/bin/ipcalc -n ${CurIP} | /usr/bin/cut -d= -f 2`
	local NewMask=`/bin/ipcalc -m ${CurIP} | /usr/bin/cut -d= -f 2`
	local NewRoute="push \"route ${NewNet} ${NewMask}\""

	if [ -n "${OldRoute}" -a -n "${NewRoute}" ]; then
		/bin/sed -i "s/${OldRoute}/${NewRoute}/g" ${OPENVPN_CONFIG} > /dev/null 2>&1
		${PACKAGE_TARGET_DIR}/scripts/openvpn.sh restart
	fi
}

case $1 in
	--sdk-mod-ver)
		echo "1.0"
		;;
	--name)
		;;
	--pkg-ver)
		;;
	--vendor)
		;;
	--pre)
		;;
	--post)
		local run=`/usr/syno/bin/get_section_key_value ${VPNPLUS_CONFIG} openvpn enable`
		local vpnifname="lbr0"
		if [ "yes" = ${run} -a ${IFNAME} = ${vpnifname} ]; then
			modify_push_route
		fi
		;;
	*)
		echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--pre|--post"
		;;
esac
