#!/bin/sh

SZF_VPNTOOL="/var/packages/VPNPlusServer/target/tool/synovpnplustool"
SZF_IFCONFIG="/sbin/ifconfig"
SZF_LOGGER="/usr/bin/logger"

is_switching_topology()
{
	if [ -f "/tmp/topology.lock" ]; then
		return 1
	fi

	return 0
}

case $1 in
	--sdk-mod-ver)
	#Print SDK support version
		echo "1.0";
	;;
	--name)
	#Print package name
		echo "VPNPlus";
	;;
	--pkg-ver)
	#Print package version
		echo "1.0";
	;;
	--vendor)
	#Print package vendor
		echo "Synology";
	;;
	--post)
		is_switching_topology
		if [ "1" = "$?" ]; then
			exit 0
		fi

		if [ "${IFNAME}" = "${IFNAME#ppp*}" ] || [ "x${IFNAME}" = "xppp200" ] || [ "x${IFNAME}" = "xppp300" ]; then
			exit 0
		fi

		if [ "${IFNAME#ppp*}" -lt 200 ]; then
			${SZF_VPNTOOL} -f -u 2>/dev/null
			exit 0
		fi

		VirtualIP=`${SZF_IFCONFIG} | grep ${IFNAME} -A 1 | tail -n 1 | awk '{print$3}' | awk -F':' '{print $2}'`
		UserData=`${SZF_VPNTOOL} -c -o | grep ${VirtualIP}`
		User=`echo ${UserData} | awk -F',' '{print $1}'`
		Type=`echo ${UserData} | awk -F',' '{print $2}'`

		if [ ${User} -eq "" ] || [ ${Type} -eq "" ]; then
			${SZF_LOGGER} -p err -t synovpnplus "synovpnplus-ppp.sh params error"
			exit 0;
		fi

		${SZF_VPNTOOL} -t -a "${User}" "${VirtualIP}" "${Type}" "${IFNAME}"

		# kick user if tc failed
		if [ $? -ne 0 ]; then
			${SZF_VPNTOOL} -f -k ${Type} ${VirtualIP}
			exit 0;
		fi

		# do sampling once
		${SZF_VPNTOOL} -t -s


		# add SNAT if the ippool of PPTP is lbr0 or gbr0
		TOPOLOGY=`/bin/get_key_value /etc/synoinfo.conf net_topology`
		IPPOOL=`/usr/syno/bin/get_section_key_value /usr/syno/etc/packages/VPNPlusServer/synovpnplus.conf pptp ippool`
		if [ "${IFNAME#ppp*}" -lt 300 ] && [ "x${IPPOOL}" = "xlbr0" -o "${IPPOOL}" = "gbr0" ]; then
			iptables -t nat -A POSTROUTING -s ${VirtualIP}/32 -j MASQUERADE
		fi
	;;
	*)
	echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--pre|--post"
	;;
esac


