#!/bin/sh

#################################################################
# This hook is invoked when topology change
#################################################################

PACKAGE_LOCATION="/var/packages/VPNPlusServer"
SZF_IPPOOL_SH="${PACKAGE_LOCATION}/target/scripts/ippool.sh"
SZF_VPNTOOL="${PACKAGE_LOCATION}/target/tool/synovpnplustool"
VPNPLUS_CONF="${PACKAGE_LOCATION}/etc/synovpnplus.conf"

add_snat() {
	if [ "${NEW_TOPOLOGY}" = "bridge" ]; then
		${PACKAGE_LOCATION}/target/scripts/ippool.sh add-all-snat
		${PACKAGE_LOCATION}/target/scripts/openvpn.sh add_snat
	fi
}

del_snat() {
	if [ "${OLD_TOPOLOGY}" = "bridge" ]; then
		${PACKAGE_LOCATION}/target/scripts/ippool.sh del-all-snat
		${PACKAGE_LOCATION}/target/scripts/openvpn.sh del_snat
	fi
}

ippool_handle() {
	pptp_ipppool=`/usr/syno/bin/get_section_key_value ${VPNPLUS_CONF} pptp ippool`
	l2tp_ipppool=`/usr/syno/bin/get_section_key_value ${VPNPLUS_CONF} l2tp ippool`
	sslvpn_ippool=`/usr/syno/bin/get_section_key_value ${VPNPLUS_CONF} sslvpn ippool`
	sstp_ippool=`/usr/syno/bin/get_section_key_value ${VPNPLUS_CONF} sstp ippool`

	if [ "${pptp_ipppool}" = "lbr0" -o "${pptp_ipppool}" = "gbr0" ]; then
		${SZF_VPNTOOL} -f -t vpnplus-pptp
	fi

	if [ "${l2tp_ipppool}" = "lbr0" -o "${l2tp_ipppool}" = "gbr0" ]; then
		${SZF_VPNTOOL} -f -t vpnplus-l2tp
	fi

	if [ "${sslvpn_ippool}" = "lbr0" -o "${sslvpn_ippool}" = "gbr0" ]; then
		${SZF_VPNTOOL} -f -t vpnplus-ssl
	fi

	if [ "${sstp_ippool}" = "lbr0" -o "${sstp_ippool}" = "gbr0" ]; then
		${SZF_VPNTOOL} -f -t vpnplus-sstp
	fi
}

case $1 in
	--sdk-mod-ver)
	#Print SDK support version
		echo "1.0";
	;;
	--name)
	#Print package name
		echo "VPNPlus";
	;;
	--pkg-ver)
	#Print package version
		echo "1.0";
	;;
	--vendor)
	#Print package vendor
		echo "Synology";
	;;
	--pre)
		del_snat
		if [ "x${NEW_TOPOLOGY}" == "xbridge" ]; then
			${SZF_IPPOOL_SH} stopTap lbr0
			${SZF_IPPOOL_SH} stopTap gbr0
		fi
	;;
	--post)
		${SZF_VPNTOOL} -t -r
		add_snat
		if [ "x${NEW_TOPOLOGY}" == "xbridge" ]; then
			ippool_handle
		fi
		if [ "x${OLD_TOPOLOGY}" == "xbridge" ]; then
			${SZF_IPPOOL_SH} startTap lbr0
			${SZF_IPPOOL_SH} startTap gbr0
		fi
		${SZF_VPNTOOL} -f -u # Auto update IF
	;;
	*)
	echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--pre|--post"
	;;
esac


